@@ -18,6 +18,8 @@ import sys
 import json
 import datetime
 
+import nuclio_sdk
+
 
 class _EventDeserializer(object):
     def deserialize(self, event_message):
@@ -28,7 +30,6 @@ class _EventDeserializer(object):
         try:
             return json.loads(body.decode("utf-8"))
         except Exception as exc:
-
             # newline to force flush
             # NOTE: processor runs sdk with `-u` which means stderr is unbuffered which needs manual flushing
             sys.stderr.write(
@@ -58,6 +59,8 @@ class _EventDeserializer(object):
             _type=parsed_data["type"],
             type_version=parsed_data["type_version"],
             version=parsed_data["version"],
+            offset=parsed_data.get("offset", 0),
+            topic=parsed_data.get("topic"),
         )
 
     @staticmethod
@@ -82,12 +85,23 @@ class _EventDeserializer(object):
             _type=parsed_data[b"type"],
             type_version=parsed_data[b"type_version"],
             version=parsed_data[b"version"],
+            offset=parsed_data.get(b"offset", 0),
+            topic=parsed_data.get(b"topic"),
         )
 
+    @staticmethod
+    def decode_single_or_list_event(parsed_data, decode_single_event_function):
+        if isinstance(parsed_data, list):
+            return [
+                decode_single_event_function(single_event_data)
+                for single_event_data in parsed_data
+            ]
+        else:
+            return decode_single_event_function(parsed_data)
+
 
 class _EventDeserializerMsgPack(_EventDeserializer):
     def __init__(self, raw=False):
-
         # return the concrete function that handled raw/decoded event messages
         # pre-assign to avoid if/else during event processing
         self._from_msgpack_handler = (
@@ -97,32 +111,40 @@ class _EventDeserializerMsgPack(_EventDeserializer):
     def deserialize(self, event_message):
         return self._from_msgpack_handler(event_message)
 
-    @staticmethod
-    def _from_msgpack_raw(parsed_data):
-        event_body = parsed_data[b"body"]
-        if parsed_data[b"content_type"] == b"application/json":
-            event_body = _EventDeserializer._try_deserialize_json(event_body)
-        return _EventDeserializer._from_parsed_data_bytes(parsed_data, event_body)
+    def _from_msgpack_raw(self, parsed_data):
+        def _decode_single_event(single_event_data):
+            event_body = single_event_data[b"body"]
+            if single_event_data[b"content_type"] == b"application/json":
+                event_body = _EventDeserializer._try_deserialize_json(event_body)
+            return _EventDeserializer._from_parsed_data_bytes(
+                single_event_data, event_body
+            )
 
-    @staticmethod
-    def _from_msgpack_decoded(parsed_data):
-        event_body = parsed_data["body"]
-        if parsed_data["content_type"] == "application/json":
-            event_body = _EventDeserializer._try_deserialize_json(event_body)
-        return _EventDeserializer._from_parsed_data(parsed_data, event_body)
+        return self.decode_single_or_list_event(parsed_data, _decode_single_event)
+
+    def _from_msgpack_decoded(self, parsed_data):
+        def _decode_single_event(single_event_data):
+            event_body = single_event_data["body"]
+            if single_event_data["content_type"] == "application/json":
+                event_body = _EventDeserializer._try_deserialize_json(event_body)
+            return _EventDeserializer._from_parsed_data(single_event_data, event_body)
+
+        return self.decode_single_or_list_event(parsed_data, _decode_single_event)
 
 
 class _EventDeserializerJSON(_EventDeserializer):
     def deserialize(self, event_message):
         parsed_data = json.loads(event_message)
 
-        # extract content type, needed to decode body
-        body = parsed_data["body"]
-        if parsed_data["content_type"] == "application/json" and not isinstance(
-            body, dict
-        ):
-            body = _EventDeserializer._try_deserialize_json(body)
-        return _EventDeserializer._from_parsed_data(parsed_data, body)
+        def _deserialize_single_event(single_event):
+            body = single_event["body"]
+            if single_event["content_type"] == "application/json" and not isinstance(
+                body, dict
+            ):
+                body = _EventDeserializer._try_deserialize_json(body)
+            return _EventDeserializer._from_parsed_data(single_event, body)
+
+        return self.decode_single_or_list_event(parsed_data, _deserialize_single_event)
 
 
 class EventDeserializerKinds(enum.Enum):
@@ -158,6 +180,7 @@ class Event(object):
         version=None,
         last_in_batch=None,
         offset=None,
+        topic=None,
     ):
         self.body = body
         self.content_type = content_type
@@ -177,9 +200,14 @@ class Event(object):
         self.version = version
         self.last_in_batch = last_in_batch or False
         self.offset = offset or 0
+        self.topic = topic
 
     def to_json(self):
-        obj = vars(self).copy()
+        obj = {}
+        for field_name, field_value in vars(self).items():
+            # exclude private fields
+            if not field_name.startswith("_"):
+                obj[field_name] = field_value
         obj["trigger"] = {
             "kind": self.trigger.kind,
             "name": self.trigger.name,
@@ -192,7 +220,7 @@ class Event(object):
         if isinstance(obj["body"], bytes):
             obj["body"] = base64.b64encode(obj["body"]).decode("ascii")
 
-        return json.dumps(obj)
+        return json.dumps(obj, default=str)
 
     def get_header(self, header_key):
         for key, value in self.headers.items():
@@ -228,15 +256,9 @@ class Event(object):
         """
         Return json of offset data
         """
-        return {
-            "kind": "streamMessageAck",
-            "attributes": {
-                "topic": self.path,
-                "partition": self.shard_id,
-                "offset": self.offset,
-                "triggerName": self.trigger.name,
-            },
-        }
+        return nuclio_sdk.QualifiedOffset.from_event(
+            self
+        ).compile_explicit_ack_message()
 
     def __repr__(self):
         return self.to_json()
