@@ -13,7 +13,6 @@
 # limitations under the License.
 
 import json
-import signal
 import http.client
 
 import nuclio_sdk
@@ -35,6 +34,8 @@ class Platform(object):
         self._connection_provider = connection_provider or http.client.HTTPConnection
 
         self._control_callback = on_control_callback
+        self._termination_callback = None
+        self._drain_callback = None
 
     async def explicit_ack(self, qualified_offset):
         """
@@ -51,16 +52,29 @@ class Platform(object):
                 "Cannot send explicit ack since control callback was not initialized"
             )
 
-    def on_signal(self, callback, sig=signal.SIGTERM):
+    def set_drain_callback(self, callback):
         """
-        Syntactic sugar to bind an incoming system signal on user's callback
+        Register a callback to be called when the platform is draining (rebalance happening in stream).
+        If already registered, the callback will be replaced.
+        When called, the callback will be called with zero arguments.
+
+        :param callback: the callback to call when terminating
         """
-        signal.signal(sig, callback)
+        self._drain_callback = callback
+
+    def set_termination_callback(self, callback):
+        """
+        Register a callback to be called when the platform is terminating.
+        If already registered, the callback will be replaced.
+        When called, the callback will be called with zero arguments.
+
+        :param callback: the callback to call when terminating
+        """
+        self._termination_callback = callback
 
     def call_function(
         self, function_name, event, node=None, timeout=None, service_name_override=None
     ):
-
         # get connection from provider
         connection = self._connection_provider(
             self._get_function_url(function_name, service_name_override),
@@ -102,7 +116,7 @@ class Platform(object):
         response_headers = {}
 
         # get response headers as lowercase
-        for (name, value) in response.getheaders():
+        for name, value in response.getheaders():
             response_headers[name.lower()] = value
 
         # if content type exists, use it
@@ -120,7 +134,6 @@ class Platform(object):
         )
 
     def _get_function_url(self, function_name, service_name_override=None):
-
         # local envs prefix namespace
         if self.kind == "local":
             service_name = service_name_override or "nuclio-{0}-{1}".format(
@@ -129,3 +142,15 @@ class Platform(object):
         else:
             service_name = service_name_override or "nuclio-{0}".format(function_name)
         return "{0}:8080".format(service_name)
+
+    def _on_signal(self, callback_type="termination"):
+        """
+        When a signal is received, call the termination/drain callback as a hook before exiting
+        If not set, the callback will be a no-op
+
+        :arg callback_type:str - callback type, can be "termination" or "drain"
+        """
+        if callback_type == "termination" and self._termination_callback:
+            return self._termination_callback()
+        elif callback_type == "drain" and self._drain_callback:
+            return self._drain_callback()
