@@ -21,11 +21,19 @@ class QualifiedOffset(object):
 
     @staticmethod
     def from_event(event):
-        return QualifiedOffset(event.topic, event.partition, event.offset)
+        # topic resolving required to keep BC (NUC-233)
+        topic = event.topic if event.topic else event.path
+        return QualifiedOffset(topic, event.shard_id, event.offset)
 
     def compile_explicit_ack_message(self):
+        """
+        Return a stream ack message with json of offset data
+        """
         return {
-            "topic": self.topic,
-            "partition": self.partition,
-            "offset": self.offset,
+            "kind": "streamMessageAck",
+            "attributes": {
+                "topic": self.topic,
+                "partition": self.partition,
+                "offset": self.offset,
+            },
         }
